import 'dart:async';

import 'package:lori_sdk/data/database/interfaces/notificationDao.dart';
import 'package:lori_sdk/data/models/notification.dart';
import 'package:lori_sdk/data/models/singleton/notificationSingleton.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';

class MobileNotificationDao implements NotificationDao {
  Database db;

  Future open() async {
    if (db == null || !db.isOpen) {
      var databasesPath = await getDatabasesPath();
      String path = join(databasesPath, 'loriNotification.db');
      db = await openDatabase(path, version: 2,
          onCreate: (Database db, int version) async {
        _createTableV2(db);
      }, onUpgrade: (Database db, int oldVersion, int newVersion) async {
        switch (oldVersion) {
          case 1:
            _upgradeTableV1toV2(db);
            break;
          default:
            break;
        }
      });
    }
  }

  void _createTableV2(Database db) {
    db.execute('''
        create table $tableName (
          $columnId integer primary key autoincrement,
          $columnTitle text,
          $columnMessage text,
          $columnNotificationId text,
          $columnImageUrl text,
          $columnIsRead integer,
          $columnAction text,
          $columnObjectReferenceId text,
          $columnOrderId text,
          $columnExternalId text
        )
    ''');
  }

  //On V2 we added the column $columnNotificationId to the database
  void _upgradeTableV1toV2(Database db) {
    db.execute('''
      alter table $tableName add $columnNotificationId text
    ''');
  }

  Future<void> insertOrUpdate(Notification notification) async {
    // Get by text
    if (notification.id == null || notification.id == 0) {
      var notif = await getByText(notification.title, notification.message);
      if (notif == null)
        return insert(notification);
      else {
        notif.title = notification.title;
        notif.message = notification.message;
        return updateByText(notif);
      }
    } else {
      // Get by id
      var notif = await getById(notification.id);
      if (notif == null)
        return insert(notification);
      else
        return update(notification);
    }
  }

  Future<void> insert(Notification notification) async {
    notification.id = null;
    await open();
    await db.insert(tableName, notification.toMap());
    var count = await get(false);
    return NotificationSingleton().notificationNumberChanged(count.length);
  }

  Future<void> update(Notification notification) async {
    await open();
    final map = notification.toMap();
    map.remove('id');
    await db.update(tableName, map,
        where: '$columnId = ?', whereArgs: [notification.id]);
    var count = await get(false);
    return NotificationSingleton().notificationNumberChanged(count.length);
  }

  Future<void> updateByText(Notification notification) async {
    await open();
    final map = notification.toMap();
    map.remove('id');
    await db.update(tableName, map,
        where: '$columnTitle = ? AND $columnMessage = ?',
        whereArgs: [notification.title, notification.message]);
    var count = await get(false);
    return NotificationSingleton().notificationNumberChanged(count.length);
  }

  Future<List<Notification>> get(bool isRead) async {
    await open();
    List<Map> maps = await db.query(tableName,
        where: '$columnIsRead = ?',
        whereArgs: [isRead ? 1 : 0],
        orderBy: '$columnId DESC');
    List<Notification> result = [];
    if (maps != null && maps.length > 0) {
      for (var map in maps) {
        var notification =
            Notification(0, '', '', '', '', false, '', '', '', '');
        notification.fromMap(map);
        result.add(notification);
      }
    }
    return result;
  }

  Future<Notification> getByText(String title, String body) async {
    await open();
    List<Map> maps = await db.query(tableName,
        where: '$columnTitle = ? AND $columnMessage = ?',
        whereArgs: [title, body]);
    if (maps != null && maps.length > 0) {
      var notification = Notification(0, '', '', '', '', false, '', '', '', '');
      notification.fromMap(maps.first);
      return notification;
    }
    return null;
  }

  Future<Notification> getById(int id) async {
    await open();
    List<Map> maps =
        await db.query(tableName, where: '$columnId = ?', whereArgs: [id]);
    if (maps != null && maps.length > 0) {
      var notification = Notification(0, '', '', '', '', false, '', '', '', '');
      notification.fromMap(maps.first);
      return notification;
    }
    return null;
  }

  Future<void> deleteAll() async {
    await open();
    return db.delete(
      tableName,
    );
  }
}

NotificationDao getNotificationDao() => MobileNotificationDao();
